/*
	Program SEGYHDR

	Program to print the SEGY headers


	The SEGY header is standard except as follows:
			trhdr[100] = MSB of sample rate in microseconds
			trhdr[101] = LSB of sample rate in microseconds
			trhdr[102]=1 ==> 32 bit integer data
			trhdr[102]=0 ==> 16 bit integer data
			trhdr[103] = start of trace in milliseconds
			trhdr[104] = trigger time year
			trhdr[105] = trigger time julian day
			trhdr[106] = trigger time hour
			trhdr[107] = trigger time minutes
			trhdr[108] = trigger time seconds
			trhdr[109] = trigger time  milliseconds
			trhdr[110] = MSW of scale factor
			trhdr[111] = LSW of scale factor 
					(amplitude = data * scale/gain)

	Written by J. C. Fowler
	    28 December 1988
*/


#include <stdio.h>	/* Standard I/O header file			*/
#include <ctype.h>
#include <string.h>
#include <fcntl.h>



char *progname;
main (argc, argv)


int		argc;
char		*argv[];

{   FILE	*fpin;		/* Pointer to the input file */
   
   int i, j, int_hold, nsam, srat, ifile, pos;
    int	    year, day, hour, min, sec, msec ;
   short trhdr[120], short_hold;
    short scf, cuf, elf, gain ;
    int  fflag, sflag, rflag, iflag ;
    long  trnl, trnr, frn, orn ;
    long  rlat, rlong, relv, slat, slong, selv ;
    long    ltemp[10], *lptr ;
   unsigned char *cptr;
   unsigned char FileIN[64], Dum[8];
   float dt, sflat, sflong, sfelv, rflat, rflong, rfelv ;
    float  scfe, scfc, scale ;

   /* take command line arguments as filesnames and flags or
      ask for the filename to be used for the conversion      */

   progname = argv[0];

for(ifile=1; ifile < argc; ifile++){
      strcpy( FileIN, &argv[ifile][0] );
      printf("\nINPUT FILE  %s \n",FileIN);

         /* open the input file */

   if (( fpin = fopen(FileIN, "r+")) == NULL ){
      printf( "\n ******  there has been an error opening %s for input  *****", FileIN);
      fclose(fpin);
      exit(1);
   }


   /*  first read the SEGY trace header    */

   cptr = ( unsigned char *) &trhdr[0];
       for ( i= 0; i< 240 ; i++){
      *cptr = fgetc(fpin);
      cptr++;
   }

/*  start processing this header */
    hgetl(&trnl,&trhdr[0]) ;	/* get trace number within line */
    hgetl(&trnr,&trhdr[2]) ;	/* trace number within reel */
    hgetl(&frn,&trhdr[4]) ;	/* original field record number */
    hgetl(&orn,&trhdr[6]) ;	/* trace number in original field record */

    printf("\nTrace Number within Line = %ld",trnl) ;
    printf("\nTrace Number within Reel = %ld",trnr) ;
    printf("\nOriginal Field Record Number = %ld",frn) ;
    printf("\nTrace Number in Original Field Record = %ld",orn) ;

/*  get sample rate and number of samples */
    hgetl(&srat,&trhdr[100]) ;	/* get sample rate */
    nsam = trhdr[57] ;		/* number of samples */
    dt = 1000000./(float)srat ;
    hgetl((long *)&scale,&trhdr[110]) ;  /* get scale factor */
    gain = trhdr[60] ;		/* get gain */

    printf("\n\nSample Rate = %ld microseconds   = %f samples/sec",srat,dt) ;
    printf("\nLength of Trace = %d samples",nsam) ;
    printf("\nGain = %d    Scale factor = %f",gain,scale) ;

/*  get position of source and receivers */
    scf = trhdr[35] ;	/* scalar for coordinates */
    cuf = trhdr[44] ;	/* type of coordinate system */
    elf = trhdr[34] ;	/* scalar for elevations */
    hgetl(&slong,&trhdr[36]) ;	/* source longitude */
    hgetl(&slat,&trhdr[38]) ;	/* source latitude */
    hgetl(&selv,&trhdr[24]) ;	/* source elevation */
    hgetl(&rlong,&trhdr[40]) ;	/* receiver longitude */
    hgetl(&rlat,&trhdr[42]) ;	/* receiver latitude */
    hgetl(&relv,&trhdr[20]) ;	/* receiver elevation */

    scfe = (float)elf ;
    if(elf < 0) scfe = -1./(float)elf ;
    scfc = (float)scf ;
    if(scf < 0) scfc = -1./(float)scf ;

    sflong = scfc * (float)slong ;
    sflat = scfc * (float)slat ;
    sfelv = scfe * (float)selv ;
    rflong = scfc * (float)rlong ;
    rflat = scfc * (float)rlat ;
    rfelv = scfe * (float)relv ;

    printf("\n\nRECEIVER POSITION ") ;
    printf("\nElevation = %f",rfelv) ;
    if(cuf == 1)
    {
	printf("\nX = %f     Y = %f",rflong,rflat) ;
    }
    else
    {
	printf("\nLat = %f     Long = %f",rflat,rflong) ;
    }
    printf("\n\nSOURCE POSITION ") ;
    printf("\nElevation = %f",sfelv) ;
    if(cuf == 1)
    {
	printf("\nX = %f     Y = %f",sflong,sflat) ;
    }
    else
    {
	printf("\nLat = %f     Long = %f",sflat,sflong) ;
    }

    /*  get trace start time */
    year = trhdr[78] ;
    day = trhdr[79] ;
    hour = trhdr[80] ;
    min = trhdr[81] ;
    sec = trhdr[82] ;
    msec = trhdr[103] ;

    printf("\n\nStart of Trace") ;
    printf(" %d  Day - %d",year,day) ;
    printf("     %02d:%02d:%02d.%03d ",hour,min,sec,msec) ; 

    /*  get event start time */
    year = trhdr[104] ;
    day = trhdr[105] ;
    hour = trhdr[106] ;
    min = trhdr[107] ;
    sec = trhdr[108] ;
    msec = trhdr[109] ;

    printf("\nStart of Event") ;
    printf(" %d  Day - %d",year,day) ;
    printf("     %02d:%02d:%02d.%03d ",hour,min,sec,msec) ; 
    
    printf("\n\n") ;
    
   fclose(fpin);

}
}


/***********************************************************************/
/*             hgetl(lptr,sptr)                                      */
/***********************************************************************/
/*	Routine to get a long variable from an array of short variables
	sptr = pointer to first position in short array
	lptr = pointer to long variable

 */

hgetl(lptr,sptr)
short	*sptr ;
long	*lptr ;
{
    
    short   *sptr1, i ;

    sptr1 = (short *)lptr ;
    for( i=0 ; i<2 ; i++,sptr++,sptr1++) *sptr1 = *sptr ;

    return ;
}
